/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.io.Serializable;
import java.util.List;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.core.Events;
import org.jboss.seam.faces.FacesMessages;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.service.IAlbumAction;

@Name("albumManager")
@Scope(ScopeType.EVENT)
@AutoCreate
public class AlbumManager implements Serializable{

	private static final long serialVersionUID = 2631634926126857691L;
	
	private boolean validationSuccess = false;
	
	@In private IAlbumAction albumAction;
	
	@In private User user;
	
	@In FacesMessages facesMessages;
	
	@Restrict("#{s:hasRole('admin')}")
	public void addAlbum(Album album){
		if(album.getShelf() == null){
			facesMessages.addToControl(Constants.SHELF_ID, Constants.SHELF_MUST_BE_NOT_NULL_ERROR, null);
			Contexts.getConversationContext().set(Constants.ALBUM_VARIABLE, album);
			return;
		}
		validationSuccess = true;
		try{
			albumAction.addAlbum(album);
		}catch(Exception e){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.ALBUM_SAVING_ERROR);
			return;
		}
		Contexts.getConversationContext().set(Constants.ALBUM_VARIABLE, null);
		Events.instance().raiseEvent(Constants.ALBUM_ADDED_EVENT, album);
		Events.instance().raiseEvent(Constants.CLEAR_TREE_EVENT);
	}
	
	@Restrict("#{s:hasRole('admin')}")
	public void createAlbum(Shelf shelf){
		Album album = new Album();
		if(shelf == null){
			if(user.getShelfs().size() > 0){
				shelf = user.getShelfs().get(0);
			}
			if(shelf == null){
				Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.NO_SHELF_ERROR);
				return;
			}
		}
		album.setShelf(shelf);
		Contexts.getConversationContext().set(Constants.ALBUM_VARIABLE, album);
	}

	@Restrict("#{s:hasRole('admin')}")
	public void editAlbum(Album album){
		try{
			albumAction.editAlbum(album);
		}catch(Exception e){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.ALBUM_SAVING_ERROR);
			return;
		}
		Events.instance().raiseEvent(Constants.ALBUM_EDITED_EVENT, album);
	}
	
	@Restrict("#{s:hasRole('admin')}")
	public void deleteAlbum(Album album){
		String pathToDelete = album.getPath();
		try{
			albumAction.deleteAlbum(album);
		}catch(Exception e){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.ALBUM_DELETING_ERROR);
			return;
		}
		Events.instance().raiseEvent(Constants.ALBUM_DELETED_EVENT, album, pathToDelete);
		Events.instance().raiseEvent(Constants.CLEAR_TREE_EVENT);
	}
	
	public List<Image> getUnvisitedImages(Album album){
		if(album.getCountUnvisitedImages() == 0){
			album.setUnvisitedImages(albumAction.getUnvisitedImages(album));
		}
		return album.getUnvisitedImages();
	}

	public boolean isValidationSuccess() {
		return validationSuccess;
	}

	public void setValidationSuccess(boolean validationSuccess) {
		this.validationSuccess = validationSuccess;
	}
}