/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.io.Serializable;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.service.Constants;

@Name("slideshow")
@Scope(ScopeType.CONVERSATION)
public class SlideshowManager implements Serializable{

	private static final long serialVersionUID = 7801877176558409702L;
	
	private Integer slideshowIndex;
	
	private Image selectedImage;
	
	private boolean active;
	
	@In Model model;
	
	private int interval = Constants.INITIAL_DELAY;

	public int getInterval() {
		return interval;
	}

	public void setInterval(int interval) {
		this.interval = interval;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}
	
	public void startSlideshow(){
		active = true;
		this.slideshowIndex = 0;
		if(model.getImages() == null || model.getImages().size() < 1){
			stopSlideshow();
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.NO_IMAGES_FOR_SLIDESHOW_ERROR);
			return;
		}
		this.selectedImage = model.getImages().get(this.slideshowIndex);
		this.selectedImage.getAlbum().visitImage(selectedImage, true);
	}
	
	public void startSlideshow(Image selectedImage){
		active = true;
		if(model.getImages() == null || model.getImages().size() < 1){
			stopSlideshow();
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.NO_IMAGES_FOR_SLIDESHOW_ERROR);
			return;
		}
		this.slideshowIndex = model.getImages().indexOf(selectedImage);
		this.selectedImage = selectedImage;
		this.selectedImage.getAlbum().visitImage(selectedImage, true);
	}
	
	@Observer(Constants.STOP_SLIDESHOW_EVENT)
	public void stopSlideshow(){
		active = false;
		this.selectedImage = null;
		this.slideshowIndex = 0;
	}

	public Integer getSlideshowIndex() {
		return slideshowIndex;
	}

	public void setSlideshowIndex(Integer slideshowIndex) {
		this.slideshowIndex = slideshowIndex;
	}

	public Image getSelectedImage() {
		return selectedImage;
	}

	public void setSelectedImage(Image selectedImage) {
		this.selectedImage = selectedImage;
	}
	
	public void showNextImage(){
		if(!active){
			return;
		}
		if(slideshowIndex == model.getImages().size() - 1){
			slideshowIndex = -1;
		}
		slideshowIndex++;
		selectedImage = model.getImages().get(slideshowIndex);
		this.selectedImage.getAlbum().visitImage(selectedImage, true);
	}
}